package db

import (
	"egg-im/internal/business/comm/db/model"
	"egg-im/internal/business/comm/md"
	"egg-im/internal/business/comm/utils/logx"
	"egg-im/pkg/db"
	"egg-im/pkg/gerrors"
	"fmt"
	"github.com/jinzhu/gorm"
)

type dbUserPushForJg struct{}

var DbUserPushForJg = new(dbUserPushForJg)

// UserPushForJgGetOne 获取一条记录
func (*dbUserPushForJg) UserPushForJgGetOne(uid string) (*model.UserPushForJg, error) {
	var cfgList model.UserPushForJg
	if err := db.DB.Where("`uid` = ?", uid).First(&cfgList).Error; err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		}
		return nil, gerrors.WrapError(err)
	}
	return &cfgList, nil
}

// UserPushForJgInsert 插入一条配置信息
func (*dbUserPushForJg) UserPushForJgInsert(uid int64, pushAlia string) bool {
	cfg := model.UserPushForJg{Uid: uid, PushAlia: pushAlia}
	err := db.DB.Create(&cfg).Error
	if err != nil {
		logx.Error(err)
		return false
	}
	return true
}

// UserPushForJgUpdate 修改某条配置信息
func (*dbUserPushForJg) UserPushForJgUpdate(uid int64, pushAlia string) bool {
	err := db.DB.Model(&model.UserPushForJg{}).Where("`uid` = ?", uid).Update("push_alia", pushAlia).Error
	if err != nil {
		logx.Error(err)
		return false
	}
	return true
}

// UserPushForJgGetWithDb 单条记录获取
func (*dbUserPushForJg) UserPushForJgGetWithDb(HKey string) string {
	cacheKey := fmt.Sprintf(md.AppUserPushForJgCacheKey, HKey[0:1])
	get, err := db.RedisUtil.HGetString(cacheKey, HKey)
	if err != nil || get == "" {
		cfg, err := DbUserPushForJg.UserPushForJgGetOne(HKey)
		if err != nil || cfg == nil {
			_ = logx.Error(err)
			return ""
		}

		// key是否存在
		cacheKeyExist := false
		if db.RedisUtil.Exists(cacheKey) {
			cacheKeyExist = true
		}

		// 设置缓存
		_, err = db.RedisUtil.HSet(cacheKey, HKey, cfg.PushAlia)
		if err != nil {
			_ = logx.Error(err)
			return ""
		}
		if !cacheKeyExist { // 如果是首次设置 设置过期时间
			_, err := db.RedisUtil.Expire(cacheKey, md.CfgCacheTime)
			if err != nil {
				_ = logx.Error(err)
				return ""
			}
		}
		return cfg.PushAlia
	}
	return get
}
